import typing

class VLPError(Exception): ...

class VLPTypeError(VLPError):
    def __init__(self, value, *typeargv: type) -> None: ...

class VLPSequenceLenError(VLPError):
    def __init__(self, value: typing.Sequence, length: int) -> None: ...

class VLPValueError(VLPError):
    def __init__(self, value, *argv) -> None: ...

class VLPValueRangeError(VLPError):
    def __init__(self, value, low, hignt) -> None: ...
