def draw_position_contrast(pred, true, title: str = ..., xlabel: str = ..., ylabel: str = ..., zlabel: str = ..., savepath=..., showfig: bool = ...) -> None: ...
def draw_history(history, title: str = ..., savepath=..., showfig: bool = ...) -> None: ...
def draw_error_hist(error, bins: int = ..., rwidth: float = ..., title: str = ..., savepath=..., showfig: bool = ...): ...
def cdfplot(error, title: str = ..., savepath=..., showfig: bool = ...): ...
