from ..io import IO as IO
from ..setting import Settings as Settings
from .vlp_model import VLP_Model as VLP_Model
from _typeshed import Incomplete

io: Incomplete
Args: Incomplete
save_excel_path: str
save_npz_path: str

def main() -> None: ...
def test_func() -> None: ...
