import numpy as np
from ..error import VLPSequenceLenError as VLPSequenceLenError, VLPValueError as VLPValueError
from numbers import Number
from typing import Union

class LED:
    def __init__(self, fov: Number = ..., theta: Number = ..., pos: tuple = ..., Nv: tuple = ..., origin: tuple = ..., signal: str = ..., *args, **kwargs) -> None: ...
    @staticmethod
    def get_time_series(fs, npt): ...
    def DC_Signal(self, Pt, *args, **kwargs): ...
    def AC_Signal(self, DCbias, Amp, freq, t, signal_func: Union[str, np.ufunc] = ..., *args, **kwargs): ...
    def AIM_Signal(self, Pt, m, freq, t, signal_func: Union[str, np.ufunc] = ..., *args, **kwargs): ...
    def send_signal(self, signal: str, *args, **kwargs): ...
    @property
    def signal(self): ...
    @property
    def origin(self) -> tuple: ...
    @property
    def pos(self) -> tuple: ...
    @property
    def Nv(self): ...
    @property
    def fov(self): ...
    @property
    def theta(self): ...
    @property
    def m(self): ...
