from typing import Sequence, Union

def signal_wave(x, *y, timeAxis: bool = ..., xlab: Union[str, Sequence] = ..., ylab: Union[str, Sequence] = ..., title: Union[str, Sequence] = ..., savepath=..., showfig: bool = ...): ...
def signal_fft_wave(x, *y, freqAxis: bool = ..., single: bool = ..., xlab: Union[str, Sequence] = ..., ylab: Union[str, Sequence] = ..., title: Union[str, Sequence] = ..., savepath=..., showfig: bool = ...): ...
