from _typeshed import Incomplete

class Settings:
    items: Incomplete
    def __init__(self, settings_path: str = ...) -> None: ...
    def set_settings_path(self, settings_path: str = ...): ...
    def get_settings_path(self) -> str: ...
    def serialize(self, filepath=...) -> None: ...
    def deserialize(self, filepath=...) -> None: ...
