from _typeshed import Incomplete

class IO:
    def save_excel(self, fp: str, *data, header: bool = ..., index: bool = ...): ...
    def load_excel(self, fp: str, sheet: str = ..., header: Incomplete | None = ...): ...
    def save_npz(self, fp: str, *data: dict): ...
    def load_npz(self, fp, **argkeys): ...
    def save_csv(self, fp: str, *dataFrame: dict): ...
    def load_csv(self, fp: str): ...
    def svae_txt(self, fp: str, data: dict): ...
    def load_txt(self, fp: str, sep: Incomplete | None = ..., header: Incomplete | None = ..., index: Incomplete | None = ...): ...
    def save_mat(self, fp: str, mdict: dict = ...): ...
    def load_mat(self, fp: str, keys: dict = ..., default: Incomplete | None = ...): ...
    def user_save_npz_2d(self, fp: str, train_p: Incomplete | None = ..., train_x: Incomplete | None = ..., train_y: Incomplete | None = ..., test_p: Incomplete | None = ..., test_x: Incomplete | None = ..., test_y: Incomplete | None = ...): ...
    def user_load_npz_2d(self, fp: str): ...
    def user_save_npz_3d(self, fp: str, train_p: Incomplete | None = ..., train_x: Incomplete | None = ..., train_y: Incomplete | None = ..., train_z: Incomplete | None = ..., test_p: Incomplete | None = ..., test_x: Incomplete | None = ..., test_y: Incomplete | None = ..., test_z: Incomplete | None = ...): ...
    def user_load_npz_3d(self, fp: str): ...
    def dataset_save_npz(self, fp: str, train_inputs: Incomplete | None = ..., train_outputs: Incomplete | None = ..., test_inputs: Incomplete | None = ..., test_outputs: Incomplete | None = ...): ...
    def dataset_load_npz(self, fp: str): ...
